<?php

declare (strict_types=1);
namespace WpifyWooDeps\DragonBe\Test\Vies;

use DomainException;
use WpifyWooDeps\DragonBe\Vies\HeartBeat;
use WpifyWooDeps\DragonBe\Vies\Vies;
use WpifyWooDeps\PHPUnit\Framework\TestCase;
/**
 * Class HeartBeatTest
 *
 * @package DragonBe\Test\Vies
 * @coversDefaultClass \DragonBe\Vies\HeartBeat
 */
class HeartBeatTest extends \WpifyWooDeps\PHPUnit\Framework\TestCase
{
    /**
     * @covers ::getHost
     */
    public function testExceptionThrownWhenNoHostIsConfigured()
    {
        $this->expectException(\DomainException::class);
        (new \WpifyWooDeps\DragonBe\Vies\HeartBeat())->getHost();
    }
    /**
     * @covers ::getPort
     */
    public function testDefaultPortIsHttp()
    {
        $hb = new \WpifyWooDeps\DragonBe\Vies\HeartBeat();
        $port = $hb->getPort();
        $this->assertSame(443, $port);
    }
    /**
     * @covers ::setHost
     * @covers ::getHost
     */
    public function testCanSetHost()
    {
        $host = 'www.example.com';
        $hb = new \WpifyWooDeps\DragonBe\Vies\HeartBeat();
        $hb->setHost($host);
        $this->assertSame($host, $hb->getHost());
    }
    /**
     * @covers ::setPort
     * @covers ::getPort
     */
    public function testCanSetPort()
    {
        $port = 443;
        $hb = new \WpifyWooDeps\DragonBe\Vies\HeartBeat();
        $hb->setPort($port);
        $this->assertSame($port, $hb->getPort());
    }
    /**
     * @covers ::setTimeout
     * @covers ::getTimeout
     */
    public function testCanSetTimeout()
    {
        $timeout = 300;
        $hb = new \WpifyWooDeps\DragonBe\Vies\HeartBeat();
        $hb->setTimeout($timeout);
        $this->assertSame($timeout, $hb->getTimeout());
    }
    /**
     * @covers ::__construct
     */
    public function testCanOverrideSettingsAtConstruct()
    {
        $host = 'www.example.com';
        $port = 8080;
        $hb = new \WpifyWooDeps\DragonBe\Vies\HeartBeat($host, $port);
        $this->assertSame($host, $hb->getHost());
        $this->assertSame($port, $hb->getPort());
    }
    /**
     * @covers ::isAlive
     */
    public function testVerifyServicesIsAlive()
    {
        $host = '127.0.0.1';
        $port = -1;
        \WpifyWooDeps\DragonBe\Vies\HeartBeat::$testingEnabled = \true;
        \WpifyWooDeps\DragonBe\Vies\HeartBeat::$testingServiceIsUp = \true;
        $hb = new \WpifyWooDeps\DragonBe\Vies\HeartBeat($host, $port);
        $this->assertTrue($hb->isAlive());
    }
    /**
     * @covers ::isAlive
     */
    public function testVerifyServicesIsDown()
    {
        $host = '127.0.0.1';
        $port = -1;
        \WpifyWooDeps\DragonBe\Vies\HeartBeat::$testingEnabled = \true;
        \WpifyWooDeps\DragonBe\Vies\HeartBeat::$testingServiceIsUp = \false;
        $hb = new \WpifyWooDeps\DragonBe\Vies\HeartBeat($host, $port);
        $this->assertFalse($hb->isAlive());
    }
    public function socketProvider() : array
    {
        return ['Non-existing socket on localhost' => ['127.0.0.1', -1, 10, \false], 'Socket 443 on ec.europe.eu' => [\WpifyWooDeps\DragonBe\Vies\Vies::VIES_DOMAIN, \WpifyWooDeps\DragonBe\Vies\Vies::VIES_PORT, 10, \true]];
    }
    /**
     * @dataProvider socketProvider
     * @covers ::isAlive
     * @covers ::reachOut
     * @covers ::getSecuredResponse
     * @covers ::readContents
     */
    public function testIsAliveUsingSockets($host, $port, $timeout, $expectedResult)
    {
        \WpifyWooDeps\DragonBe\Vies\HeartBeat::$testingEnabled = \false;
        $heartBeat = new \WpifyWooDeps\DragonBe\Vies\HeartBeat($host, $port, $timeout);
        $actualResult = $heartBeat->isAlive();
        $this->assertSame($expectedResult, $actualResult);
    }
}
