<?php

declare (strict_types=1);
namespace WpifyWooDeps\h4kuna\Ares\Tests;

use WpifyWooDeps\Salamium\Testinium;
use WpifyWooDeps\Tester\Assert;
use WpifyWooDeps\h4kuna\Ares;
require_once __DIR__ . '/../bootstrap.php';
/**
 * @testCase
 */
class AresTest extends \WpifyWooDeps\h4kuna\Ares\Tests\TestCase
{
    /**
     * @throws \h4kuna\Ares\Exceptions\IdentificationNumberNotFoundException
     */
    public function testNotExists() : void
    {
        (new \WpifyWooDeps\h4kuna\Ares\Ares())->loadData('36620751');
    }
    public function testFreelancer() : void
    {
        $ares = new \WpifyWooDeps\h4kuna\Ares\Ares();
        $in = '87744473';
        /* @var $data Ares\Data */
        $data = (string) $ares->loadData($in);
        // Testinium\File::save($in . '.json', (string) $data);
        \WpifyWooDeps\Tester\Assert::same(\WpifyWooDeps\Salamium\Testinium\File::load($in . '.json'), $data);
    }
    public function testMerchant() : void
    {
        $ares = new \WpifyWooDeps\h4kuna\Ares\Ares();
        $in = '27082440';
        /* @var $data Ares\Data */
        $data = (string) $ares->loadData($in);
        // Testinium\File::save($in . '.json', (string) $data);
        \WpifyWooDeps\Tester\Assert::same(\WpifyWooDeps\Salamium\Testinium\File::load($in . '.json'), $data);
    }
    public function testMerchantInActive() : void
    {
        $ares = new \WpifyWooDeps\h4kuna\Ares\Ares();
        $in = '25596641';
        /* @var $data Ares\Data */
        $data = \json_encode($ares->loadData($in));
        // Testinium\File::save($in . '.json', (string) $data);
        \WpifyWooDeps\Tester\Assert::same(\WpifyWooDeps\Salamium\Testinium\File::load($in . '.json'), $data);
    }
    public function testHouseNumber() : void
    {
        $ares = new \WpifyWooDeps\h4kuna\Ares\Ares();
        $in = '26713250';
        /* @var $data Ares\Data */
        $data = \json_encode($ares->loadData($in));
        // Testinium\File::save($in . '.json', (string) $data);
        \WpifyWooDeps\Tester\Assert::same(\WpifyWooDeps\Salamium\Testinium\File::load($in . '.json'), $data);
    }
    public function testToArray() : void
    {
        $ares = new \WpifyWooDeps\h4kuna\Ares\Ares();
        $data = $ares->loadData('87744473');
        \WpifyWooDeps\Tester\Assert::same('Milan Matějček', $data->company);
        $names = [];
        foreach (self::allPropertyRead($data) as $value) {
            if (!\preg_match('~\\$(?P<name>.*)~', $value, $find)) {
                throw new \RuntimeException('Bad annotation property-read od Data class: ' . $value);
            }
            \WpifyWooDeps\Tester\Assert::true($data->exists($find['name']));
            $names[$find['name']] = \true;
        }
        \WpifyWooDeps\Tester\Assert::same([], \array_diff_key($data->getData(), $names));
        \WpifyWooDeps\Tester\Assert::same(["620", "461", "471", "73110", "7490"], $data->nace);
        \WpifyWooDeps\Tester\Assert::type('array', $data->toArray());
        \WpifyWooDeps\Tester\Assert::same(['c' => 'Milan Matějček', 'company' => \true, 'city' => 'Mladá Boleslav'], $data->toArray(['company' => 'c', 'is_person' => 'company', 'city' => null]));
    }
    /**
     * @throws \h4kuna\Ares\Exceptions\IdentificationNumberNotFoundException
     */
    public function testNoIn() : void
    {
        (new \WpifyWooDeps\h4kuna\Ares\Ares())->loadData('123');
    }
    public function testForeingPerson() : void
    {
        $data = (new \WpifyWooDeps\h4kuna\Ares\Ares())->loadData('6387446');
        \WpifyWooDeps\Tester\Assert::true($data->is_person);
    }
    /**
     * @return array<string>
     */
    private static function allPropertyRead(\WpifyWooDeps\h4kuna\Ares\Data $data) : array
    {
        $doc = (new \ReflectionClass($data))->getDocComment();
        if ($doc === \false) {
            throw new \RuntimeException();
        }
        \preg_match_all('/@property-read *(?P<propertyRead>.+)/', $doc, $match);
        return $match['propertyRead'];
    }
    public function testLoadByIdentificationNumbers() : void
    {
        $identificationNumbers = ['6387446', '123', '87744473', '25596641'];
        $results = (new \WpifyWooDeps\h4kuna\Ares\Ares())->loadByIdentificationNumbers($identificationNumbers);
        \WpifyWooDeps\Tester\Assert::count(2, $results[\WpifyWooDeps\h4kuna\Ares\Ares::RESULT_FAILED]);
        \WpifyWooDeps\Tester\Assert::count(2, $results[\WpifyWooDeps\h4kuna\Ares\Ares::RESULT_SUCCESS]);
        \WpifyWooDeps\Tester\Assert::same(['c' => 'Ivan Šebesta', 'company' => \true, 'city' => 'Břest'], $results[\WpifyWooDeps\h4kuna\Ares\Ares::RESULT_SUCCESS][0]->toArray(['company' => 'c', 'is_person' => 'company', 'city' => null]));
        \WpifyWooDeps\Tester\Assert::same(['c' => 'Milan Matějček', 'company' => \true, 'city' => 'Mladá Boleslav'], $results[\WpifyWooDeps\h4kuna\Ares\Ares::RESULT_SUCCESS][2]->toArray(['company' => 'c', 'is_person' => 'company', 'city' => null]));
        \WpifyWooDeps\Tester\Assert::same(['in' => '123', 'code' => 0, 'message' => 'Chyba 71 - nenalezeno 123'], $results[\WpifyWooDeps\h4kuna\Ares\Ares::RESULT_FAILED][1]->toArray());
        \WpifyWooDeps\Tester\Assert::same(['in' => '25596641', 'code' => 0, 'message' => 'Chyba 61 - subjekt zanikl'], $results[\WpifyWooDeps\h4kuna\Ares\Ares::RESULT_FAILED][3]->toArray());
        \WpifyWooDeps\Tester\Assert::same(0, $results[\WpifyWooDeps\h4kuna\Ares\Ares::RESULT_FAILED][3]->getCode());
        \WpifyWooDeps\Tester\Assert::same('25596641', $results[\WpifyWooDeps\h4kuna\Ares\Ares::RESULT_FAILED][3]->getIn());
        \WpifyWooDeps\Tester\Assert::same('Chyba 61 - subjekt zanikl', $results[\WpifyWooDeps\h4kuna\Ares\Ares::RESULT_FAILED][3]->getMessage());
    }
}
(new \WpifyWooDeps\h4kuna\Ares\Tests\AresTest())->run();
