<?php

namespace WpifyWooDeps\Heureka;

use WpifyWooDeps\Heureka\ShopCertification\IRequester;
use WpifyWooDeps\Mockery;
require_once __DIR__ . '/../vendor/autoload.php';
/**
 * @author Jakub Chábek <jakub.chabek@heureka.cz>
 */
class ShopCertificationTest extends \WpifyWooDeps\PHPUnit_Framework_TestCase
{
    public function tearDown()
    {
        \WpifyWooDeps\Mockery::close();
    }
    public function testLogSetOrderWithInvalidOrderId()
    {
        $requester = \WpifyWooDeps\Mockery::mock('WpifyWooDeps\\Heureka\\ShopCertification\\IRequester');
        $requester->shouldReceive('setApiEndpoint')->once()->with(\WpifyWooDeps\Mockery::type('WpifyWooDeps\\Heureka\\ShopCertification\\ApiEndpoint'));
        $shopCertification = new \WpifyWooDeps\Heureka\ShopCertification('xxxxxxxx', [], $requester);
        $this->setExpectedException('WpifyWooDeps\\Heureka\\ShopCertification\\InvalidArgumentException');
        $shopCertification->setOrderId('abcd');
    }
    public function testLogOrderSuccess()
    {
        $requester = \WpifyWooDeps\Mockery::mock('WpifyWooDeps\\Heureka\\ShopCertification\\IRequester');
        $requester->shouldReceive('setApiEndpoint')->once()->with(\WpifyWooDeps\Mockery::type('WpifyWooDeps\\Heureka\\ShopCertification\\ApiEndpoint'));
        $response = \WpifyWooDeps\Mockery::mock('WpifyWooDeps\\Heureka\\ShopCertification\\Response');
        $response->code = 200;
        $response->message = 'ok';
        $postData = ['apiKey' => $apiKey = 'xxxxxxxxxx', 'email' => $email = 'john@doe.com', 'orderId' => $orderId = 12345, 'productItemIds' => [$product1 = 'ab12345', $product2 = '123459']];
        $requester->shouldReceive('request')->once()->with(\WpifyWooDeps\Heureka\ShopCertification\IRequester::ACTION_LOG_ORDER, \WpifyWooDeps\Mockery::mustBe([]), \WpifyWooDeps\Mockery::mustBe($postData))->andReturn($response);
        $shopCertification = new \WpifyWooDeps\Heureka\ShopCertification($apiKey, [], $requester);
        $shopCertification->setEmail($email);
        $shopCertification->setOrderId($orderId);
        $shopCertification->addProductItemId($product1);
        $shopCertification->addProductItemId($product2);
        $result = $shopCertification->logOrder();
        $this->assertInstanceOf('WpifyWooDeps\\Heureka\\ShopCertification\\Response', $result);
    }
    public function testLogOrderSuccessWithoutOptionalFields()
    {
        $requester = \WpifyWooDeps\Mockery::mock('WpifyWooDeps\\Heureka\\ShopCertification\\IRequester');
        $requester->shouldReceive('setApiEndpoint')->once()->with(\WpifyWooDeps\Mockery::type('WpifyWooDeps\\Heureka\\ShopCertification\\ApiEndpoint'));
        $response = \WpifyWooDeps\Mockery::mock('WpifyWooDeps\\Heureka\\ShopCertification\\Response');
        $response->code = 200;
        $response->message = 'ok';
        $postData = ['apiKey' => $apiKey = 'xxxxxxxxxx', 'email' => $email = 'john@doe.com'];
        $requester->shouldReceive('request')->once()->with(\WpifyWooDeps\Heureka\ShopCertification\IRequester::ACTION_LOG_ORDER, \WpifyWooDeps\Mockery::mustBe([]), \WpifyWooDeps\Mockery::mustBe($postData))->andReturn($response);
        $shopCertification = new \WpifyWooDeps\Heureka\ShopCertification($apiKey, [], $requester);
        $shopCertification->setEmail($email);
        $result = $shopCertification->logOrder();
        $this->assertInstanceOf('WpifyWooDeps\\Heureka\\ShopCertification\\Response', $result);
    }
    public function testLogOrderMissingEmail()
    {
        $requester = \WpifyWooDeps\Mockery::mock('WpifyWooDeps\\Heureka\\ShopCertification\\IRequester');
        $requester->shouldReceive('setApiEndpoint')->once()->with(\WpifyWooDeps\Mockery::type('WpifyWooDeps\\Heureka\\ShopCertification\\ApiEndpoint'));
        $shopCertification = new \WpifyWooDeps\Heureka\ShopCertification('xxxxxxxxxx', [], $requester);
        $this->setExpectedException('WpifyWooDeps\\Heureka\\ShopCertification\\MissingInformationException');
        $shopCertification->logOrder();
    }
    public function testLogOrderDoubleSend()
    {
        $requester = \WpifyWooDeps\Mockery::mock('WpifyWooDeps\\Heureka\\ShopCertification\\IRequester');
        $requester->shouldReceive('setApiEndpoint')->once()->with(\WpifyWooDeps\Mockery::type('WpifyWooDeps\\Heureka\\ShopCertification\\ApiEndpoint'));
        $response = \WpifyWooDeps\Mockery::mock('WpifyWooDeps\\Heureka\\ShopCertification\\Response');
        $response->code = 200;
        $response->message = 'ok';
        $postData = ['apiKey' => $apiKey = 'xxxxxxxxxx', 'email' => $email = 'john@doe.com', 'productItemIds' => [$product1 = 'ab12345', $product2 = '123459']];
        $requester->shouldReceive('request')->once()->with(\WpifyWooDeps\Heureka\ShopCertification\IRequester::ACTION_LOG_ORDER, \WpifyWooDeps\Mockery::mustBe([]), \WpifyWooDeps\Mockery::mustBe($postData))->andReturn($response);
        $shopCertification = new \WpifyWooDeps\Heureka\ShopCertification($apiKey, [], $requester);
        $shopCertification->setEmail($email);
        $shopCertification->addProductItemId($product1);
        $shopCertification->addProductItemId($product2);
        $shopCertification->logOrder();
        $this->setExpectedException('WpifyWooDeps\\Heureka\\ShopCertification\\Exception');
        $shopCertification->logOrder();
    }
}
