<?php

declare (strict_types=1);
/*
 * This file is part of the Monolog package.
 *
 * (c) Jordi Boggiano <j.boggiano@seld.be>
 *
 * For the full copyright and license information, please view the LICENSE
 * file that was distributed with this source code.
 */
namespace WpifyWooDeps\Monolog\Test;

use WpifyWooDeps\Monolog\Logger;
use WpifyWooDeps\Monolog\DateTimeImmutable;
use WpifyWooDeps\Monolog\Formatter\FormatterInterface;
/**
 * Lets you easily generate log records and a dummy formatter for testing purposes
 * *
 * @author Jordi Boggiano <j.boggiano@seld.be>
 */
class TestCase extends \WpifyWooDeps\PHPUnit\Framework\TestCase
{
    /**
     * @return array Record
     */
    protected function getRecord($level = \WpifyWooDeps\Monolog\Logger::WARNING, $message = 'test', array $context = []) : array
    {
        return ['message' => (string) $message, 'context' => $context, 'level' => $level, 'level_name' => \WpifyWooDeps\Monolog\Logger::getLevelName($level), 'channel' => 'test', 'datetime' => new \WpifyWooDeps\Monolog\DateTimeImmutable(\true), 'extra' => []];
    }
    protected function getMultipleRecords() : array
    {
        return [$this->getRecord(\WpifyWooDeps\Monolog\Logger::DEBUG, 'debug message 1'), $this->getRecord(\WpifyWooDeps\Monolog\Logger::DEBUG, 'debug message 2'), $this->getRecord(\WpifyWooDeps\Monolog\Logger::INFO, 'information'), $this->getRecord(\WpifyWooDeps\Monolog\Logger::WARNING, 'warning'), $this->getRecord(\WpifyWooDeps\Monolog\Logger::ERROR, 'error')];
    }
    protected function getIdentityFormatter() : \WpifyWooDeps\Monolog\Formatter\FormatterInterface
    {
        $formatter = $this->createMock(\WpifyWooDeps\Monolog\Formatter\FormatterInterface::class);
        $formatter->expects($this->any())->method('format')->will($this->returnCallback(function ($record) {
            return $record['message'];
        }));
        return $formatter;
    }
}
