<?php

declare (strict_types=1);
namespace WpifyWooDeps\DI\Definition\Resolver;

use WpifyWooDeps\DI\Definition\Definition;
use WpifyWooDeps\DI\Definition\InstanceDefinition;
use WpifyWooDeps\DI\DependencyException;
use WpifyWooDeps\Psr\Container\NotFoundExceptionInterface;
/**
 * Injects dependencies on an existing instance.
 *
 * @since 5.0
 * @author Matthieu Napoli <matthieu@mnapoli.fr>
 */
class InstanceInjector extends \WpifyWooDeps\DI\Definition\Resolver\ObjectCreator
{
    /**
     * Injects dependencies on an existing instance.
     *
     * @param InstanceDefinition $definition
     */
    public function resolve(\WpifyWooDeps\DI\Definition\Definition $definition, array $parameters = [])
    {
        try {
            $this->injectMethodsAndProperties($definition->getInstance(), $definition->getObjectDefinition());
        } catch (\WpifyWooDeps\Psr\Container\NotFoundExceptionInterface $e) {
            $message = \sprintf('Error while injecting dependencies into %s: %s', \get_class($definition->getInstance()), $e->getMessage());
            throw new \WpifyWooDeps\DI\DependencyException($message, 0, $e);
        }
        return $definition;
    }
    public function isResolvable(\WpifyWooDeps\DI\Definition\Definition $definition, array $parameters = []) : bool
    {
        return \true;
    }
}
