<?php

namespace WpifyWooDeps\WpifyCustomFields\Implementations;

use WpifyWooDeps\WpifyCustomFields\WpifyCustomFields;
/**
 * Class ProductOptions
 * @package WpifyCustomFields\Implementations
 */
final class ProductOptions extends \WpifyWooDeps\WpifyCustomFields\Implementations\AbstractPostImplementation
{
    /** @var int */
    private $product_id;
    /** @var array */
    private $tab;
    /** @var array */
    private $items;
    /** @var bool */
    private $is_new_tab = \false;
    /**
     * ProductOptions constructor.
     *
     * @param array $args
     * @param WpifyCustomFields $wcf
     */
    public function __construct(array $args, \WpifyWooDeps\WpifyCustomFields\WpifyCustomFields $wcf)
    {
        parent::__construct($args, $wcf);
        /*
         * Possible classes: hide_if_grouped, show_if_simple, show_if_variable, show_if_grouped,
         * show_if_external, hide_if_external, hide_if_grouped, hide_if_virtual
         */
        $args = wp_parse_args($args, array('product_id' => null, 'tab' => array('id' => 'general', 'label' => null, 'priority' => 100, 'target' => null, 'class' => array()), 'items' => array()));
        $this->tab = $args['tab'];
        $this->items = $this->prepare_items($args['items']);
        $this->product_id = $args['product_id'];
        if (empty($this->tab['target'])) {
            $this->tab['target'] = $this->tab['id'] . '_product_data';
        }
        if (empty($this->tab['priority'])) {
            $this->tab['priority'] = 100;
        }
        if (empty($this->tab['class'])) {
            $this->tab['class'] = array();
        }
        add_filter('woocommerce_product_data_tabs', array($this, 'woocommerce_product_data_tabs'), 98);
        add_action('woocommerce_product_data_panels', array($this, 'render_data_panels'));
        add_action('woocommerce_product_options_' . $this->tab['target'], array($this, 'render_custom_fields'));
        add_action('woocommerce_process_product_meta', array($this, 'save'));
        add_action('init', array($this, 'register_meta'));
    }
    public function register_meta()
    {
        foreach ($this->get_items() as $item) {
            $sanitizer = $this->sanitizer->get_sanitizer($item);
            register_post_meta('product', $item['id'], array('single' => \true, 'sanitize_callback' => $sanitizer));
        }
    }
    public function get_items()
    {
        $items = apply_filters('wcf_product_options_items', $this->items, array('tab' => $this->tab));
        return $this->prepare_items($items);
    }
    /**
     * @return void
     */
    public function set_wcf_shown()
    {
        global $pagenow;
        $current_screen = get_current_screen();
        $this->wcf_shown = $current_screen->base === 'post' && $current_screen->post_type === 'product' && \in_array($pagenow, array('post-new.php', 'post.php'));
    }
    /**
     * @param array $tabs
     *
     * @return array
     */
    public function woocommerce_product_data_tabs(array $tabs)
    {
        if (isset($tabs[$this->tab['id']])) {
            if (!empty($this->tab['label'])) {
                $tabs[$this->tab['id']]['label'] = $this->tab['label'];
            }
            if (!empty($this->tab['priority'])) {
                $tabs[$this->tab['id']]['priority'] = $this->tab['priority'];
            }
        } else {
            $this->is_new_tab = \true;
            $tabs[$this->tab['id']] = array('label' => $this->tab['label'], 'priority' => $this->tab['priority'], 'target' => $this->tab['target'], 'class' => $this->tab['class']);
        }
        return $tabs;
    }
    /**
     * @return void
     */
    public function render_data_panels()
    {
        if ($this->is_new_tab) {
            ?>
			<div id="<?php 
            echo esc_attr($this->tab['target']);
            ?>" class="panel woocommerce_options_panel">
				<?php 
            do_action('woocommerce_product_options_' . $this->tab['target']);
            ?>
			</div>
			<?php 
        }
    }
    /**
     * @return void
     */
    public function render_custom_fields()
    {
        global $post;
        $this->set_post($post->ID);
        $this->render_fields();
    }
    /**
     * @param number $post_id
     */
    public function set_post($post_id)
    {
        $this->product_id = $post_id;
    }
    /**
     * @return array
     */
    public function get_data()
    {
        return array('object_type' => 'product_options', 'tab' => $this->tab, 'items' => $this->items);
    }
    /**
     * @param string $name
     *
     * @return mixed
     */
    public function get_field($name)
    {
        return get_post_meta($this->product_id, $name, \true);
    }
    /**
     * @param number $post_id
     */
    public function save($post_id)
    {
        $this->set_post($post_id);
        foreach ($this->items as $item) {
            $value = $_POST[$item['id']];
            $this->set_field($item['id'], $value);
        }
    }
    /**
     * @param string $name
     * @param string $value
     *
     * @return bool|int
     */
    public function set_field($name, $value)
    {
        return update_post_meta($this->product_id, $name, $value);
    }
}
