<?php

namespace WpifyWooDeps\WpifyCustomFields;

use WpifyWooDeps\WpifyCustomFields\Implementations\GutenbergBlock;
use WpifyWooDeps\WpifyCustomFields\Implementations\Metabox;
use WpifyWooDeps\WpifyCustomFields\Implementations\Options;
use WpifyWooDeps\WpifyCustomFields\Implementations\ProductOptions;
use WpifyWooDeps\WpifyCustomFields\Implementations\Taxonomy;
use WpifyWooDeps\WpifyCustomFields\Implementations\WooCommerceSettings;
/**
 * Class WpifyCustomFields
 * @package WpifyCustomFields
 */
final class WpifyCustomFields
{
    /** @var Assets */
    private $assets;
    /** @var Sanitizer */
    private $sanitizer;
    /** @var Parser */
    private $parser;
    /** @var Api */
    private $api;
    /**
     * WpifyCustomFields constructor.
     */
    public function __construct(string $wcf_url = '')
    {
        $assets_path = \realpath(__DIR__ . '/../build');
        $this->assets = new \WpifyWooDeps\WpifyCustomFields\Assets($assets_path, $wcf_url);
        $this->sanitizer = new \WpifyWooDeps\WpifyCustomFields\Sanitizer();
        $this->parser = new \WpifyWooDeps\WpifyCustomFields\Parser();
        $this->api = new \WpifyWooDeps\WpifyCustomFields\Api();
    }
    /**
     * @param array $args
     *
     * @return Options
     */
    public function add_options_page($args = array())
    {
        return new \WpifyWooDeps\WpifyCustomFields\Implementations\Options($args, $this);
    }
    /**
     * @param array $args
     *
     * @return Metabox
     */
    public function add_metabox($args = array())
    {
        return new \WpifyWooDeps\WpifyCustomFields\Implementations\Metabox($args, $this);
    }
    /**
     * @param array $args
     *
     * @return ProductOptions
     */
    public function add_product_options($args = array())
    {
        return new \WpifyWooDeps\WpifyCustomFields\Implementations\ProductOptions($args, $this);
    }
    /**
     * @param array $args
     *
     * @return Taxonomy
     */
    public function add_taxonomy_options($args = array())
    {
        return new \WpifyWooDeps\WpifyCustomFields\Implementations\Taxonomy($args, $this);
    }
    /**
     * @param array $args
     *
     * @return WooCommerceSettings
     */
    public function add_woocommerce_settings($args = array())
    {
        return new \WpifyWooDeps\WpifyCustomFields\Implementations\WooCommerceSettings($args, $this);
    }
    public function add_gutenberg_block($args = array())
    {
        return new \WpifyWooDeps\WpifyCustomFields\Implementations\GutenbergBlock($args, $this);
    }
    /**
     * @return Parser
     */
    public function get_parser() : \WpifyWooDeps\WpifyCustomFields\Parser
    {
        return $this->parser;
    }
    /**
     * @return Sanitizer
     */
    public function get_sanitizer() : \WpifyWooDeps\WpifyCustomFields\Sanitizer
    {
        return $this->sanitizer;
    }
    /**
     * @return Api
     */
    public function get_api() : \WpifyWooDeps\WpifyCustomFields\Api
    {
        return $this->api;
    }
    /**
     * @return Assets
     */
    public function get_assets() : \WpifyWooDeps\WpifyCustomFields\Assets
    {
        return $this->assets;
    }
}
